package jez04.structure.test;

import static org.junit.jupiter.api.Assertions.assertTrue;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.PrintStream;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.net.URISyntaxException;
import java.net.URL;
import java.nio.file.FileVisitResult;
import java.nio.file.FileVisitor;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.attribute.BasicFileAttributes;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.reflections.Configuration;
import org.reflections.Reflections;
import org.reflections.scanners.Scanners;
import org.reflections.util.ConfigurationBuilder;

import javafx.event.ActionEvent;
import javafx.fxml.FXML;

class ClassStructureTest {

	StructureHelper helper = new StructureHelper();

	@Test
	void gameControllerExistenceTest() {
		helper.classExist("GameController");
	}

	@Test
	void gameControllerFxmlTest() {
		helper.classExist("GameController");
		Class<?> c = helper.getClass("GameController");
		helper.hasPropertyWithAnnotation(c, ".*", FXML.class);
	}

	@Test
	void gameControllerActionMethodTest() {
		helper.classExist("GameController");
		Class<?> c = helper.getClass("GameController");
		helper.hasMethodRegexp(c, ".*", void.class, ActionEvent.class);
	}

	@Test
	void gameControllerLambdasTest() {
		helper.classExist("GameController");
		Class<?> c = helper.getClass("GameController");
		long lamdaCount = helper.countMethodRegexp(c, "lambda\\$.*");
		long innerClasscount = helper.countClassesRegexp(".*GameController\\$.*");
		assertTrue(lamdaCount + innerClasscount >= 3,
				"At least 3 inner classes or lamdas required for GameController but only "
						+ (lamdaCount + innerClasscount) + " found.");
	}

	@Test
	void deadListenerExistenceTest() {
		helper.classExist("DeadListener");
	}

	@Test
	void deadListenerEventMethodTest() {
		helper.classExist("DeadListener");
		Class<?> c = helper.getClass("DeadListener");
		helper.hasMethod(c, "monsterDead");
	}

	@Test
	void sceneCollectionTest() {
		helper.classExist("Level");
		Class<?> c = helper.getClass("Level");
		long collectionCount = Arrays.asList(c.getDeclaredFields()).stream()
				.filter(f -> Collection.class.isAssignableFrom(f.getType())).count();
		assertTrue(collectionCount >= 3, "lab.Scene require atleast 3 filed of type/subtype Collection, but only "
				+ collectionCount + " found.");
	}

	@Test
	void sceneMethodAddTest() {
		helper.classExist("Level");
		Class<?> c = helper.getClass("Level");
		helper.hasMethod(c, "add", void.class, helper.getClass("DrawableSimulable"));
		;
	}

	@Test
	void levelMethodRemoveTest() {
		helper.classExist("Level");
		Class<?> c = helper.getClass("Level");
		helper.hasMethod(c, "remove", void.class, helper.getClass("DrawableSimulable"));
		;
	}

	@Test
	void monsterMethodAddTest() {
		helper.classExist("Monster");
		Class<?> c = helper.getClass("Monster");
		helper.hasMethod(c, "addDeadListener");
	}

	@Test
	void monsterMethodRemoveTest() {
		helper.classExist("Monster");
		Class<?> c = helper.getClass("Monster");
		Class<?> l = helper.getClass("DeadListener");
		helper.hasMethodRegexp(c, "remove.*", List.of(void.class, boolean.class), l);
	}

	@Test
	void monsterMethodFireTest() {
		helper.classExist("Monster");
		Class<?> c = helper.getClass("Monster");
		assertTrue(helper.countMethodRegexp(c, "fire.*") > 0, "Method fire.* in LochNess not found.");
	}

	@Test
	void zIndexMethodTest() {
		helper.classExist("DrawableSimulable");
		Class<?> c = helper.getClass("DrawableSimulable");
		helper.hasMethodRegexp(c, "get[zZ][iI]ndex", int.class, new Class[0]);
	}

	@Test
	void zIndexFieldTest() {
		helper.classExist("WorldEntity");
		Class<?> c = helper.getClass("WorldEntity");
		helper.hasProperty(c, "[zZ][iI]ndex", int.class, false);
	}
}
